package sun.management.snmp.jvminstr;

import com.sun.jmx.snmp.SnmpVarBind;
import com.sun.jmx.snmp.SnmpString;
import com.sun.jmx.snmp.SnmpCounter64;
import java.lang.management.MemoryNotificationInfo;
import javax.management.openmbean.CompositeData;
import javax.management.Notification;
import javax.management.NotificationListener;
import java.lang.management.MemoryPoolMXBean;
import sun.management.snmp.util.SnmpCachedData;
import com.sun.jmx.snmp.agent.SnmpMibTable;
import sun.management.snmp.util.SnmpTableHandler;
import sun.management.snmp.jvmmib.JvmCompilationMeta;
import sun.management.snmp.jvmmib.JvmRuntimeMeta;
import sun.management.snmp.jvmmib.JvmThreadingMeta;
import sun.management.snmp.jvmmib.JvmMemoryMeta;
import javax.management.MBeanServer;
import javax.management.ObjectName;
import java.util.List;
import javax.management.ListenerNotFoundException;
import java.util.Iterator;
import com.sun.jmx.snmp.SnmpParameters;
import com.sun.jmx.snmp.SnmpPeer;
import com.sun.jmx.snmp.daemon.SnmpAdaptorServer;
import com.sun.jmx.snmp.SnmpVarBindList;
import com.sun.jmx.snmp.SnmpOid;
import java.lang.management.ManagementFactory;
import sun.management.snmp.jvmmib.JVM_MANAGEMENT_MIBOidTable;
import javax.management.NotificationEmitter;
import java.util.ArrayList;
import com.sun.jmx.snmp.SnmpOidTable;
import java.lang.ref.WeakReference;
import sun.management.snmp.util.MibLogger;
import sun.management.snmp.jvmmib.JVM_MANAGEMENT_MIB;

public class JVM_MANAGEMENT_MIB_IMPL extends JVM_MANAGEMENT_MIB
{
    private static final long serialVersionUID = -8104825586888859831L;
    private static final MibLogger log;
    private static WeakReference<SnmpOidTable> tableRef;
    private ArrayList<NotificationTarget> notificationTargets;
    private final NotificationEmitter emitter;
    private final JVM_MANAGEMENT_MIB_IMPL.NotificationHandler handler;
    private static final int DISPLAY_STRING_MAX_LENGTH = 255;
    private static final int JAVA_OBJECT_NAME_MAX_LENGTH = 1023;
    private static final int PATH_ELEMENT_MAX_LENGTH = 1023;
    private static final int ARG_VALUE_MAX_LENGTH = 1023;
    private static final int DEFAULT_CACHE_VALIDITY_PERIOD = 1000;
    
    public static SnmpOidTable getOidTable() {
        if (JVM_MANAGEMENT_MIB_IMPL.tableRef == null) {
            JVM_MANAGEMENT_MIBOidTable referent2 = new JVM_MANAGEMENT_MIBOidTable();
            JVM_MANAGEMENT_MIB_IMPL.tableRef = new WeakReference(referent2);
            return referent2;
        }
        SnmpOidTable referent = (SnmpOidTable)JVM_MANAGEMENT_MIB_IMPL.tableRef.get();
        if (referent == null) {
            referent = new JVM_MANAGEMENT_MIBOidTable();
            JVM_MANAGEMENT_MIB_IMPL.tableRef = new WeakReference(referent);
            return referent;
        }
        return referent;
    }
    
    public JVM_MANAGEMENT_MIB_IMPL() {
        this.notificationTargets = new ArrayList();
        this.handler = new JVM_MANAGEMENT_MIB_IMPL.NotificationHandler();
        (this.emitter = (NotificationEmitter)ManagementFactory.getMemoryMXBean()).addNotificationListener(this.handler, null, null);
    }
    
    private synchronized void sendTrap(SnmpOid snmpOid, SnmpVarBindList list) {
        Iterator<NotificationTarget> iterator = this.notificationTargets.iterator();
        SnmpAdaptorServer snmpAdaptorServer = (SnmpAdaptorServer)this.getSnmpAdaptor();
        if (snmpAdaptorServer == null) {
            JVM_MANAGEMENT_MIB_IMPL.log.error("sendTrap", "Cannot send trap: adaptor is null.");
            return;
        }
        if (!snmpAdaptorServer.isActive()) {
            JVM_MANAGEMENT_MIB_IMPL.log.config("sendTrap", "Adaptor is not active: trap not sent.");
            return;
        }
        while (iterator.hasNext()) {
            NotificationTarget obj = null;
            try {
                obj = (NotificationTarget)iterator.next();
                SnmpPeer o = new SnmpPeer(obj.getAddress(), obj.getPort());
                SnmpParameters params = new SnmpParameters();
                params.setRdCommunity(obj.getCommunity());
                o.setParams(params);
                JVM_MANAGEMENT_MIB_IMPL.log.debug("handleNotification", "Sending trap to " + obj.getAddress() + ":" + obj.getPort());
                snmpAdaptorServer.snmpV2Trap(o, snmpOid, list, null);
            }
            catch (Exception obj2) {
                JVM_MANAGEMENT_MIB_IMPL.log.error("sendTrap", "Exception occurred while sending trap to [" + obj + "]. Exception : " + obj2);
                JVM_MANAGEMENT_MIB_IMPL.log.debug("sendTrap", obj2);
            }
        }
        return;
    }
    
    public synchronized void addTarget(NotificationTarget e) throws IllegalArgumentException {
        if (e == null) {
            throw new IllegalArgumentException("Target is null");
        }
        this.notificationTargets.add(e);
        return;
    }
    
    public void terminate() {
        try {
            this.emitter.removeNotificationListener(this.handler);
        }
        catch (ListenerNotFoundException obj) {
            JVM_MANAGEMENT_MIB_IMPL.log.error("terminate", "Listener Not found : " + obj);
        }
    }
    
    public synchronized void addTargets(List<NotificationTarget> c) throws IllegalArgumentException {
        if (c == null) {
            throw new IllegalArgumentException("Target list is null");
        }
        this.notificationTargets.addAll(c);
        return;
    }
    
    protected Object createJvmMemoryMBean(String s, String s2, ObjectName objectName, MBeanServer mBeanServer) {
        if (mBeanServer != null) {
            return new JvmMemoryImpl(this, mBeanServer);
        }
        return new JvmMemoryImpl(this);
    }
    
    protected JvmMemoryMeta createJvmMemoryMetaNode(String s, String s2, ObjectName objectName, MBeanServer mBeanServer) {
        return new JvmMemoryMetaImpl(this, this.objectserver);
    }
    
    protected JvmThreadingMeta createJvmThreadingMetaNode(String s, String s2, ObjectName objectName, MBeanServer mBeanServer) {
        return new JvmThreadingMetaImpl(this, this.objectserver);
    }
    
    protected Object createJvmThreadingMBean(String s, String s2, ObjectName objectName, MBeanServer mBeanServer) {
        if (mBeanServer != null) {
            return new JvmThreadingImpl(this, mBeanServer);
        }
        return new JvmThreadingImpl(this);
    }
    
    protected JvmRuntimeMeta createJvmRuntimeMetaNode(String s, String s2, ObjectName objectName, MBeanServer mBeanServer) {
        return new JvmRuntimeMetaImpl(this, this.objectserver);
    }
    
    protected Object createJvmRuntimeMBean(String s, String s2, ObjectName objectName, MBeanServer mBeanServer) {
        if (mBeanServer != null) {
            return new JvmRuntimeImpl(this, mBeanServer);
        }
        return new JvmRuntimeImpl(this);
    }
    
    protected JvmCompilationMeta createJvmCompilationMetaNode(String s, String s2, ObjectName objectName, MBeanServer mBeanServer) {
        if (ManagementFactory.getCompilationMXBean() == null) {
            return null;
        }
        return super.createJvmCompilationMetaNode(s, s2, objectName, mBeanServer);
    }
    
    protected Object createJvmCompilationMBean(String s, String s2, ObjectName objectName, MBeanServer mBeanServer) {
        if (mBeanServer != null) {
            return new JvmCompilationImpl(this, mBeanServer);
        }
        return new JvmCompilationImpl(this);
    }
    
    protected Object createJvmOSMBean(String s, String s2, ObjectName objectName, MBeanServer mBeanServer) {
        if (mBeanServer != null) {
            return new JvmOSImpl(this, mBeanServer);
        }
        return new JvmOSImpl(this);
    }
    
    protected Object createJvmClassLoadingMBean(String s, String s2, ObjectName objectName, MBeanServer mBeanServer) {
        if (mBeanServer != null) {
            return new JvmClassLoadingImpl(this, mBeanServer);
        }
        return new JvmClassLoadingImpl(this);
    }
    
    static String validDisplayStringTC(String s) {
        if (s == null) {
            return "";
        }
        if (s.length() > 255) {
            return s.substring(0, 255);
        }
        return s;
    }
    
    static String validJavaObjectNameTC(String s) {
        if (s == null) {
            return "";
        }
        if (s.length() > 1023) {
            return s.substring(0, 1023);
        }
        return s;
    }
    
    static String validPathElementTC(String s) {
        if (s == null) {
            return "";
        }
        if (s.length() > 1023) {
            return s.substring(0, 1023);
        }
        return s;
    }
    
    static String validArgValueTC(String s) {
        if (s == null) {
            return "";
        }
        if (s.length() > 1023) {
            return s.substring(0, 1023);
        }
        return s;
    }
    
    private SnmpTableHandler getJvmMemPoolTableHandler(Object o) {
        SnmpMibTable registeredTableMeta = this.getRegisteredTableMeta("JvmMemPoolTable");
        if (!(registeredTableMeta instanceof JvmMemPoolTableMetaImpl)) {
            JVM_MANAGEMENT_MIB_IMPL.log.error("getJvmMemPoolTableHandler", (registeredTableMeta == null) ? "No metadata for JvmMemPoolTable" : ("Bad metadata class for JvmMemPoolTable: " + registeredTableMeta.getClass().getName()));
            return null;
        }
        return ((JvmMemPoolTableMetaImpl)registeredTableMeta).getHandler(o);
    }
    
    private int findInCache(SnmpTableHandler snmpTableHandler, String s) {
        if (!(snmpTableHandler instanceof SnmpCachedData)) {
            if (snmpTableHandler != null) {
                JVM_MANAGEMENT_MIB_IMPL.log.error("getJvmMemPoolEntry", "Bad class for JvmMemPoolTable datas: " + snmpTableHandler.getClass().getName());
            }
            return -1;
        }
        else {
            SnmpCachedData snmpCachedData = (SnmpCachedData)snmpTableHandler;
            int length = snmpCachedData.datas.length;
            int i = 0;
            while (i < snmpCachedData.datas.length) {
                if (s.equals(((MemoryPoolMXBean)snmpCachedData.datas[i]).getName())) {
                    return i;
                }
                ++i;
            }
            return -1;
        }
    }
    
    private SnmpOid getJvmMemPoolEntryIndex(SnmpTableHandler snmpTableHandler, String s) {
        int inCache = this.findInCache(snmpTableHandler, s);
        if (inCache < 0) {
            return null;
        }
        return ((SnmpCachedData)snmpTableHandler).indexes[inCache];
    }
    
    private SnmpOid getJvmMemPoolEntryIndex(String s) {
        return this.getJvmMemPoolEntryIndex(this.getJvmMemPoolTableHandler(null), s);
    }
    
    public long validity() {
        return 1000L;
    }
    
    static SnmpOid access$100(JVM_MANAGEMENT_MIB_IMPL jvm_MANAGEMENT_MIB_IMPL, String s) {
        return jvm_MANAGEMENT_MIB_IMPL.getJvmMemPoolEntryIndex(s);
    }
    
    static void access$200(JVM_MANAGEMENT_MIB_IMPL jvm_MANAGEMENT_MIB_IMPL, SnmpOid snmpOid, SnmpVarBindList list) {
        jvm_MANAGEMENT_MIB_IMPL.sendTrap(snmpOid, list);
    }
    
    static {
        log = new MibLogger(JVM_MANAGEMENT_MIB_IMPL.class);
    }
    
    private class NotificationHandler implements NotificationListener
    {
        @Override
        public void handleNotification(Notification notification, Object o) {
            JVM_MANAGEMENT_MIB_IMPL.log.debug("handleNotification", "Received notification [ " + notification.getType() + "]");
            String type = notification.getType();
            if (type.equals("java.management.memory.threshold.exceeded") || type.equals("java.management.memory.collection.threshold.exceeded")) {
                MemoryNotificationInfo from = MemoryNotificationInfo.from((CompositeData)notification.getUserData());
                SnmpCounter64 snmpCounter64 = new SnmpCounter64(from.getCount());
                SnmpCounter64 snmpCounter65 = new SnmpCounter64(from.getUsage().getUsed());
                SnmpString snmpString = new SnmpString(from.getPoolName());
                SnmpOid access$100 = JVM_MANAGEMENT_MIB_IMPL.access$100(JVM_MANAGEMENT_MIB_IMPL.this, from.getPoolName());
                if (access$100 == null) {
                    JVM_MANAGEMENT_MIB_IMPL.log.error("handleNotification", "Error: Can't find entry index for Memory Pool: " + from.getPoolName() + ": No trap emitted for " + type);
                    return;
                }
                SnmpOid snmpOid = null;
                SnmpOidTable oidTable = JVM_MANAGEMENT_MIB_IMPL.getOidTable();
                try {
                    SnmpOid snmpOid2 = null;
                    SnmpOid snmpOid3 = null;
                    if (type.equals("java.management.memory.threshold.exceeded")) {
                        snmpOid = new SnmpOid(oidTable.resolveVarName("jvmLowMemoryPoolUsageNotif").getOid());
                        snmpOid2 = new SnmpOid(oidTable.resolveVarName("jvmMemPoolUsed").getOid() + "." + access$100);
                        snmpOid3 = new SnmpOid(oidTable.resolveVarName("jvmMemPoolThreshdCount").getOid() + "." + access$100);
                    }
                    else if (type.equals("java.management.memory.collection.threshold.exceeded")) {
                        snmpOid = new SnmpOid(oidTable.resolveVarName("jvmLowMemoryPoolCollectNotif").getOid());
                        snmpOid2 = new SnmpOid(oidTable.resolveVarName("jvmMemPoolCollectUsed").getOid() + "." + access$100);
                        snmpOid3 = new SnmpOid(oidTable.resolveVarName("jvmMemPoolCollectThreshdCount").getOid() + "." + access$100);
                    }
                    SnmpVarBindList o2 = new SnmpVarBindList();
                    SnmpOid o3 = new SnmpOid(oidTable.resolveVarName("jvmMemPoolName").getOid() + "." + access$100);
                    SnmpVarBind o4 = new SnmpVarBind(snmpOid3, snmpCounter64);
                    SnmpVarBind o5 = new SnmpVarBind(snmpOid2, snmpCounter65);
                    o2.add(new SnmpVarBind(o3, snmpString));
                    o2.add(o4);
                    o2.add(o5);
                    JVM_MANAGEMENT_MIB_IMPL.access$200(JVM_MANAGEMENT_MIB_IMPL.this, snmpOid, o2);
                }
                catch (Exception obj) {
                    JVM_MANAGEMENT_MIB_IMPL.log.error("handleNotification", "Exception occurred : " + obj);
                }
            }
        }
    }
}
